/* Program to read IP addresses from a file and produce a list of 
   distinct addresses and a count of how many times each  appeared in 
   the file. The addresses and counts are stored in a linked list.

   Input (keyboard): name of file containing addresses
   Input (file): addresses

   Output: a list of distinct addresses and their counts
--------------------------------------------------------------------*/

#include <cassert>                  // assert
#include <string>                   // string
#include <iostream>                 // cin, cout, >>, <<
#include <iomanip>                  // setw()
#include <fstream>                  // ifstream, isopen()
#include <list>                     // list<T>
#include <algorithm>                // find
using namespace std;

//--------------- Begin class AddressItem -------------------------
class AddressCounter
{
 public:
   void read(istream & in) { in >> address; count = 1; }

   void print(ostream & out) const
   { out << setw(15) << left << address 
         << " occurs " << count << " times\n"; }

   void tally() { count++; }

   bool operator==(const AddressCounter & addr2)
   { return address == addr2.address; }

   string getAddress() const
   { return address; }

 private:
   string address;
   int count;
};
//----------------- End class AddressCounter --------------------

typedef list<AddressCounter> IP_List;

int main()
{
   string fileName;                       // file of IP addresses
   IP_List addrCountList;                 // list of addresses

   ifstream inStream;                     // open file of addresses
   cout << "Enter name of file containing IP addresses: ";
   cin >> fileName;
   inStream.open(fileName.data());
   assert(inStream.is_open());

   AddressCounter item;                   // one address & its count
   for (;;)                               // input loop:
   {
      item.read(inStream);                //   read an address
      if (inStream.eof()) break;          //   if eof, quit

      IP_List::iterator it =              //   is item in list?
        find(addrCountList.begin(), addrCountList.end(), item);
      if (it != addrCountList.end())      //   if so:
        (*it).tally();                    //    ++ its count
      else                                // otherwise
        addrCountList.push_back(item);    // add it to the list
   }                                      // end loop

   cout << "\nAddresses and Counts:\n\n";    // output the list
   for (IP_List::iterator it = addrCountList.begin();
                          it != addrCountList.end(); it++)
      (*it).print(cout);
}
